@@ -2,6 +2,7 @@ package main
 
 import (
 	"bufio"
+	"encoding/base64"
 	"fmt"
 	"io"
 	"io/ioutil"
@@ -11,13 +12,14 @@ import (
 	"time"
 
 	"github.com/Shopify/sarama"
-	"github.com/burdiyan/kafkautil"
+	"github.com/birdayz/kaf/pkg/partitioner"
 	pb "github.com/golang/protobuf/proto"
 	"github.com/spf13/cobra"
 )
 
 var (
 	keyFlag         string
+	rawKeyFlag      bool
 	headerFlag      []string
 	repeatFlag      int
 	partitionerFlag string
@@ -25,12 +27,14 @@ var (
 	partitionFlag   int32
 	bufferSizeFlag  int
 	inputModeFlag   string
+	avroSchemaID    int
 )
 
 func init() {
 	rootCmd.AddCommand(produceCmd)
 
 	produceCmd.Flags().StringVarP(&keyFlag, "key", "k", "", "Key for the record. Currently only strings are supported.")
+	produceCmd.Flags().BoolVar(&rawKeyFlag, "raw-key", false, "Treat value of --key as base64 and use its decoded raw value as key")
 	produceCmd.Flags().StringArrayVarP(&headerFlag, "header", "H", []string{}, "Header in format <key>:<value>. May be used multiple times to add more headers.")
 	produceCmd.Flags().IntVarP(&repeatFlag, "repeat", "n", 1, "Repeat records to send.")
 
@@ -43,6 +47,8 @@ func init() {
 	produceCmd.Flags().StringVar(&timestampFlag, "timestamp", "", "Select timestamp for record")
 	produceCmd.Flags().Int32VarP(&partitionFlag, "partition", "p", -1, "Partition to produce to")
 
+	produceCmd.Flags().IntVarP(&avroSchemaID, "avro-schema-id", "", -1, "Value schema id for avro messsage encoding")
+
 	produceCmd.Flags().StringVarP(&inputModeFlag, "input-mode", "", "line", "Scanning input mode: [line|full]")
 	produceCmd.Flags().IntVarP(&bufferSizeFlag, "line-length-limit", "", 0, "line length limit in line input mode")
 }
@@ -82,7 +88,7 @@ var produceCmd = &cobra.Command{
 		cfg := getConfig()
 		switch partitionerFlag {
 		case "jvm":
-			cfg.Producer.Partitioner = kafkautil.NewJVMCompatiblePartitioner
+			cfg.Producer.Partitioner = partitioner.NewJVMCompatiblePartitioner
 		case "rand":
 			cfg.Producer.Partitioner = sarama.NewRandomPartitioner
 		case "rr":
@@ -107,7 +113,15 @@ var produceCmd = &cobra.Command{
 		}
 
 		var key sarama.Encoder
-		key = sarama.StringEncoder(keyFlag)
+		if rawKeyFlag {
+			keyBytes, err := base64.RawStdEncoding.DecodeString(keyFlag)
+			if err != nil {
+				errorExit("--raw-key is given, but value of --key is not base64")
+			}
+			key = sarama.ByteEncoder(keyBytes)
+		} else {
+			key = sarama.StringEncoder(keyFlag)
+		}
 		if keyProtoType != "" {
 			if dynamicMessage := reg.MessageForType(keyProtoType); dynamicMessage != nil {
 				err = dynamicMessage.UnmarshalJSON([]byte(keyFlag))
@@ -127,6 +141,13 @@ var produceCmd = &cobra.Command{
 
 		}
 
+		if avroSchemaID != -1 {
+			schemaCache = getSchemaCache()
+			if schemaCache == nil {
+				errorExit("Error getting a instance of schemaCache")
+			}
+		}
+
 		var headers []sarama.RecordHeader
 		for _, h := range headerFlag {
 			v := strings.SplitN(h, ":", 2)
@@ -155,6 +176,12 @@ var produceCmd = &cobra.Command{
 				} else {
 					errorExit("Failed to load payload proto type")
 				}
+			} else if avroSchemaID != -1 {
+				avro, err := schemaCache.EncodeMessage(avroSchemaID, data)
+				if err != nil {
+					errorExit("Failed to encode avro", err)
+				}
+				data = avro
 			}
 
 			var ts time.Time
